@@ -16,14 +16,27 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id$ */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "php.h"
+
+#if HAVE_SSE_CRC32
+#include "Zend/zend_cpuinfo.h"
+#include <nmmintrin.h>
+static uint32_t crc32c_sse42(const char *dagta, unsigned int size);
+#endif
+
+#include "yac_atomic.h"
 #include "yac_storage.h"
 #include "allocator/yac_allocator.h"
 
 yac_storage_globals *yac_storage;
 
+static uint32_t (*yac_crc)(const char *data, unsigned int size);
+static uint32_t crc32(const char *dagta, unsigned int size);
+
 static inline unsigned int yac_storage_align_size(unsigned int size) /* {{{ */ {
 	int bits = 0;
 	while ((size = size >> 1)) {
@@ -39,7 +52,14 @@ int yac_storage_startup(unsigned long fsize, unsigned long size, char **msg) /*
 	if (!yac_allocator_startup(fsize, size, msg)) {
 		return 0;
 	}
-
+#if HAVE_SSE_CRC32
+	if (zend_cpu_supports_sse42()) {
+		yac_crc = crc32c_sse42;
+	} else
+#endif
+	{
+		yac_crc = crc32;
+	}
 	size = YAC_SG(first_seg).size - ((char *)YAC_SG(slots) - (char *)yac_storage);
 	real_size = yac_storage_align_size(size / sizeof(yac_kv_key));
 	if (!((size / sizeof(yac_kv_key)) & ~(real_size << 1))) {
@@ -67,7 +87,7 @@ void yac_storage_shutdown(void) /* {{{ */ {
 
 /* {{{ MurmurHash2 (Austin Appleby)
  */
-static inline uint64_t yac_inline_hash_func1(char *data, unsigned int len) {
+static inline uint64_t yac_inline_hash_func1(const char *data, unsigned int len) {
     unsigned int h, k;
 
     h = 0 ^ len;
@@ -139,7 +159,7 @@ static inline uint64_t yac_inline_hash_func1(char *data, unsigned int len) {
  *                  -- Ralf S. Engelschall <rse@engelschall.com>
  */
 
-static inline uint64_t yac_inline_hash_func2(char *key, uint32_t len) {
+static inline uint64_t yac_inline_hash_func2(const char *key, uint32_t len) {
 	register uint64_t hash = 5381;
 
 	/* variant with the hash unrolled eight times */
@@ -255,22 +275,51 @@ static unsigned int crc32_tab[] = {
 	0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
 };
 
-static inline unsigned int crc32(char *buf, unsigned int size) {
+static uint32_t crc32(const char *buf, unsigned int size) {
 	const char *p;
-	register int crc = 0;
+	uint32_t crc = 0 ^ 0xFFFFFFFF;
 
 	p = buf;
 	while (size--) {
 		crc = crc32_tab[(crc ^ *p++) & 0xFF] ^ (crc >> 8);
 	}
 
-	return crc ^ ~0U;
+	return crc ^ 0xFFFFFFFF;
 }
 /* }}} */
 
+#if HAVE_SSE_CRC32
+static uint32_t crc32c_sse42(const char *buf, unsigned int size) /* {{{ */ {
+	uint32_t crc = 0 ^ 0xFFFFFFFF;
+#if __x86_64__
+	while (size >= sizeof(uint64_t)) {
+		crc = _mm_crc32_u64(crc, *(uint64_t*)buf);
+		buf += sizeof(uint64_t);
+		size -= sizeof(uint64_t);
+	}
+#endif
+	while (size >= sizeof(uint32_t)) {
+		crc = _mm_crc32_u32(crc, *(uint32_t*)buf);
+		buf += sizeof(uint32_t);
+		size -= sizeof(uint32_t);
+	}
+	if (size >= sizeof(uint16_t)) {
+		crc = _mm_crc32_u16(crc, *(uint16_t*)buf);
+		buf += sizeof(uint16_t);
+		size -= sizeof(uint16_t);
+	}
+	if (size) {
+		crc = _mm_crc32_u8(crc, *buf);
+	}
+
+	return crc ^ 0xFFFFFFFF;
+}
+/* }}} */
+#endif
+
 static inline unsigned int yac_crc32(char *data, unsigned int size) /* {{{ */ {
 	if (size < YAC_FULL_CRC_THRESHOLD) {
-		return crc32(data, size);
+		return yac_crc(data, size);
 	} else {
 		int i = 0;
 		char crc_contents[YAC_FULL_CRC_THRESHOLD];
@@ -287,19 +336,24 @@ static inline unsigned int yac_crc32(char *data, unsigned int size) /* {{{ */ {
 		}
 		memcpy(q, p, tail);
 
-		return crc32(crc_contents, YAC_FULL_CRC_THRESHOLD);
+		return yac_crc(crc_contents, YAC_FULL_CRC_THRESHOLD);
 	}
 }
 /* }}} */
 
-int yac_storage_find(char *key, unsigned int len, char **data, unsigned int *size, unsigned int *flag, int *cas, unsigned long tv) /* {{{ */ {
+int yac_storage_find(const char *key, unsigned int len, char **data, unsigned int *size, unsigned int *flag, int *cas, unsigned long tv) /* {{{ */ {
 	uint64_t h, hash, seed;
 	yac_kv_key k, *p;
 	yac_kv_val v;
 
 	hash = h = yac_inline_hash_func1(key, len);
 	p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+	if (!WRITEP(p)) {
+		++YAC_SG(miss);
+		return 0;
+	}
 	k = *p;
+	READP(p);
 	if (k.val) {
 		char *s;
 		uint32_t i;
@@ -342,7 +396,12 @@ do_verify:
 		for (i = 0; i < 3; i++) {
 			h += seed & YAC_SG(slots_mask);
 			p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+			if (!WRITEP(p)) {
+				++YAC_SG(miss);
+				return 0;
+			}
 			k = *p;
+			READP(p);
 			if (k.h == hash && YAC_KEY_KLEN(k) == len) {
 				v = *(k.val);
 				if (!memcmp(k.key, key, len)) {
@@ -360,13 +419,17 @@ do_verify:
 }
 /* }}} */
 
-void yac_storage_delete(char *key, unsigned int len, int ttl, unsigned long tv) /* {{{ */ {
+int yac_storage_delete(const char *key, unsigned int len, int ttl, unsigned long tv) /* {{{ */ {
 	uint64_t hash, h, seed;
 	yac_kv_key k, *p;
 
 	hash = h = yac_inline_hash_func1(key, len);
 	p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+	if (!WRITEP(p)) {
+		return 0;
+	}
 	k = *p;
+	READP(p);
 	if (k.val) {
 		uint32_t i;
 		if (k.h == hash && YAC_KEY_KLEN(k) == len) {
@@ -376,7 +439,7 @@ void yac_storage_delete(char *key, unsigned int len, int ttl, unsigned long tv)
 				} else {
 					p->ttl = ttl + tv;
 				}
-				return;
+				return 1;
 			}
 		} 
 
@@ -384,19 +447,25 @@ void yac_storage_delete(char *key, unsigned int len, int ttl, unsigned long tv)
 		for (i = 0; i < 3; i++) {
 			h += seed & YAC_SG(slots_mask);
 			p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+			if (!WRITEP(p)) {
+				return 0;
+			}
 			k = *p;
+			READP(p);
 			if (k.val == NULL) {
-				return;
+				return 1;
 			} else if (k.h == hash && YAC_KEY_KLEN(k) == len && !memcmp((char *)k.key, key, len)) {
 				p->ttl = 1;
-				return;
+				return 1;
 			}
 		}
 	}
+
+	return 0;
 }
 /* }}} */
 
-int yac_storage_update(char *key, unsigned int len, char *data, unsigned int size, unsigned int flag, int ttl, int add, unsigned long tv) /* {{{ */ {
+int yac_storage_update(const char *key, unsigned int len, char *data, unsigned int size, unsigned int flag, int ttl, int add, unsigned long tv) /* {{{ */ {
 	uint64_t hash, h;
 	int idx = 0, is_valid;
 	yac_kv_key *p, k, *paths[4];
@@ -405,7 +474,11 @@ int yac_storage_update(char *key, unsigned int len, char *data, unsigned int siz
 
 	hash = h = yac_inline_hash_func1(key, len);
 	paths[idx++] = p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+	if (!WRITEP(p)) {
+		return 0;
+	}
 	k = *p;
+	READP(p);
 	if (k.val) {
 		/* Found the exact match */
 		if (k.h == hash && YAC_KEY_KLEN(k) == len && !memcmp((char *)k.key, key, len)) {
@@ -432,7 +505,12 @@ do_update:
 				k.flag = flag;
 				memcpy(k.key, key, len);
 				YAC_KEY_SET_LEN(k, len, size);
+				if (!WRITEP(p)) {
+					USER_FREE(s);
+					return 0;
+				}
 				*p = k;
+				READP(p);
 				USER_FREE(s);
 				return 1;
 			} else {
@@ -461,7 +539,12 @@ do_update:
 					k.size = real_size;
 					memcpy(k.key, key, len);
 					YAC_KEY_SET_LEN(k, len, size);
+					if (!WRITEP(p)) {
+						USER_FREE(s);
+						return 0;
+					}
 					*p = k;
+					READP(p);
 					USER_FREE(s);
 					return 1;
 				}
@@ -477,7 +560,11 @@ do_update:
 			for (i = 0; i < 3; i++) {
 				h += seed & YAC_SG(slots_mask);
 				paths[idx++] = p = &(YAC_SG(slots)[h & YAC_SG(slots_mask)]);
+				if (!WRITEP(p)) {
+					return 0;
+				}
 				k = *p;
+				READP(p);
 				if (k.val == NULL) {
 					goto do_add;
 				} else if (k.h == hash && YAC_KEY_KLEN(k) == len && !memcmp((char *)k.key, key, len)) {
@@ -497,8 +584,12 @@ do_update:
 					p = paths[i];
 				}
 			}
-			++YAC_SG(kicks);
+			if (!WRITEP(p)) {
+				return 0;
+			}
 			k = *p;
+			READP(p);
+			++YAC_SG(kicks);
 			k.h = hash;
 
 			goto do_update;
@@ -532,20 +623,25 @@ do_add:
 			} else {
 				k.ttl = 0;
 			}
+			if (!WRITEP(p)) {
+				USER_FREE(s);
+				return 0;
+			}
 			*p = k;
+			READP(p);
 			USER_FREE(s);
 			return 1;
 		}
 		++YAC_SG(fails);
 		USER_FREE(s);
 	}
-
 	return 0;
 }
 /* }}} */
 
 void yac_storage_flush(void) /* {{{ */ {
 	YAC_SG(slots_num) = 0;
+
 	memset((char *)YAC_SG(slots), 0, sizeof(yac_kv_key) * YAC_SG(slots_size));
 }
 /* }}} */
